local SEND_IN_PROGRESS = "%d mails to send, please wait ..."
local SEND_DONE = "Done sending mails."
local SEND_ERROR = "Not sending your mails for some reason."
local INVALID_RANK = "Rank %q is not a valid guild rank."
local ERR_NOT_AT_MAILBOX = "You are not at the mailbox."
local ERR_SENDING = "Postmaster is already processing a mail queue, please wait."
local ERR_RECIPIENT = "Failed sending the mail to %s."
local ERR_MAIL_UNAVAILABLE = "Mailbox suddenly unavailable in the middle of sending mails, stopping."

local addon = CreateFrame("Frame")
local guildRanks = {}
local guildMembers = {}
local queue = {}
local mailAvailable = nil
local pName = UnitName("player")

local currentRecipient = nil
local subject = nil
local body = nil
local waiting = nil

local function processQueue(self)
	if waiting then return end
	if #queue > 0 then
		if not mailAvailable then
			print(ERR_MAIL_UNAVAILABLE)
			self:Hide()
			return
		end
		currentRecipient = table.remove(queue)
		waiting = true
		SendMail(currentRecipient, subject, body)
		if #queue == 0 then
			self:Hide()
			print(SEND_DONE)
		end
	end
end

local function processNext(success)
	if not waiting then return end
	waiting = nil
	if not success then
		print(ERR_RECIPIENT:format(currentRecipient or "?"))
	end
end

function addon:MAIL_SEND_SUCCESS() processNext(true) end
function addon:MAIL_FAILED() processNext(false) end

addon:SetScript("OnEvent", function(self, event, ...) self[event](self, ...) end)
function addon:GUILD_ROSTER_UPDATE()
	wipe(guildRanks)
	wipe(guildMembers)
	for i = 1, GuildControlGetNumRanks() do
		guildRanks[(GuildControlGetRankName(i))] = i
	end
	for i = 1, GetNumGuildMembers(true) do
		local n, _, r = GetGuildRosterInfo(i)
		if n and n ~= pName then guildMembers[n] = r + 1 end
	end
end
function addon:MAIL_CLOSED()
	wipe(queue)
	mailAvailable = nil
end
function addon:MAIL_SHOW()
	if IsInGuild() then GuildRoster() end
	mailAvailable = true
end
addon:RegisterEvent("MAIL_SEND_SUCCESS")
addon:RegisterEvent("MAIL_FAILED")
addon:RegisterEvent("GUILD_ROSTER_UPDATE")
addon:RegisterEvent("MAIL_CLOSED")
addon:RegisterEvent("MAIL_SHOW")
addon:Hide()
addon:SetScript("OnUpdate", processQueue)
addon:SetScript("OnHide", function()
	wipe(queue)
	currentRecipient = nil
	subject = nil
	body = nil
	waiting = nil
end)

_G.SlashCmdList["POSTMASTER_SHORTHAND"] = function(input)
	if not mailAvailable then
		print(ERR_NOT_AT_MAILBOX)
		return
	end
	if #queue > 1 then
		print(ERR_SENDING)
		return
	end
	local andUp = nil
	if input:find("+$") then
		andUp = true
		input = input:sub(1, #input - 1)
	end
	if not guildRanks[input] then
		print(INVALID_RANK:format(input))
		return
	end
	wipe(queue)
	local rankIndex = guildRanks[input]
	for name, rank in pairs(guildMembers) do
		if (andUp and rank <= rankIndex) or (not andUp and rank == rankIndex) then
			queue[#queue + 1] = name
		end
	end

	subject = SendMailSubjectEditBox:GetText():trim()
	body = SendMailBodyEditBox:GetText():trim()
	if #queue > 0 and subject and subject:len() > 3 and body and body:len() > 3 then
		currentRecipient = nil
		print(SEND_IN_PROGRESS:format(#queue))
		addon:Show()
	else
		subject = nil
		body = nil
		wipe(queue)
		print(SEND_ERROR)
	end
end
_G.SLASH_POSTMASTER_SHORTHAND1 = "/postmaster"

